#!/usr/bin/env python
# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from pycalphad import Database, equilibrium, variables as v

'''
   FeO enthalpy vs. temperature calculation
   feo_kje_sun4.tdb is from:
 
   10Kje Thermodynamic assessment of the Fe-Mn-O system L.Kjellqvist,
         M.Selleby, J. Phase Equilibria, 31 (2010), 113-134.

   just slightly modified for compatibility with OC6
'''

p0 = 101325.0 # Pa
T0 = 298.15 # K

def fPlothFeO():
 vx=1.0/2.0
 stdb = "feo_kje_sun4.tdb"
 dbf = Database(stdb)
 comps = list(dbf.elements)
 phases = list(dbf.phases.keys())
 comps=[ 'FE','O','VA']
 eq = equilibrium(dbf, comps, phases,
                  {v.P: p0,
                   v.T: (290.0,2500,1),
                   v.X('FE'): vx
                   },
                   output=['HM'])
 T=eq['T'].values
 h=eq['HM'].values.squeeze()
 plt.plot(T,h/1.0e3,label="pycalphad",linestyle="solid",linewidth=2)
 #
 lloc01=fReadplt(sf="plt-hm-feo-01.plt")
 lloc02=fReadplt(sf="plt-hm-feo-02.plt")
 lloc03=fReadplt(sf="plt-hm-feo-03.plt")
 lloc=np.concatenate((lloc01,lloc02,lloc03),axis=1)
 plt.plot(lloc[0],lloc[1]/1.0e3,label="OC6",linewidth=3,linestyle="dotted")
 #
 plt.legend()
 plt.grid()
 plt.xlabel("Temperature [K]")
 plt.ylabel("Enthalpy [kJ/mol]")
 plt.tight_layout()
 plt.show()
 plt.close()
 #
pass 

def fFMissE(s):
 '''  
    auxiliary function to deal with missing E in number in OC6 output 
 ''' 
 if "E" in s:
  return float(s)
 else:
  if "." in s:
   ls=s.split(".")
   sp=""
   if "+" in ls[1]: sp="+"
   if "-" in ls[1]: sp="-"
   if sp=="" :
    return float(s)
   else:
    lls=ls[1].split(sp)
    sep="E%s" % (sp)
    ss=sep.join(lls)
    return float(".".join([ls[0],ss]))
  else:
   return float(s)
  pass
 pass 
pass

def fReadplt(sf="plt-mu.plt"):
 '''
    read data from OC plt file
 '''
 n=0   
 f=open(sf,"r")
 for line in f:
  s=line.strip()
  if len(s)==0 : continue
  if s[0]=="#": continue
  if s=="EOD": break
  ls=s.split()
  if n>1 :
   for i in range(n):
    ll[i].append(fFMissE(ls[i+1]))
   pass
   continue    
  pass    
  if ls[0]=="KEYS:" :
   ll=[]
   n=len(ls)-1
   for i in range(n):
    ll.append([])
   pass 
  pass
 pass
 f.close()
 al=np.array(ll)
 als=al[:,al[0,:].argsort()]
 return als
pass

if __name__ == "__main__":
 fPlothFeO()
pass 
